#include <stdint.h>
#include <stdbool.h>
#include "Config.h"
#include "stm32f107.h"

extern uint32_t __text_start;
extern uint32_t __text_end;
extern uint32_t __data_start;
extern uint32_t __data_end;
extern uint32_t __bss_start;
extern uint32_t __bss_end;
extern uint32_t __stack_end;
extern int main();

typedef void (* const voidfunc)();

void __Reset();
void __weak __NMI();
void __weak __HardFault();
void __weak __MemManageFault();
void __weak __BusFault();
void __weak __UsageFault();
void __weak __SVCall();
void __weak __DebugMonitor();
void __weak __PendSV();
void __weak __SysTick();

void __Default();

__attribute__((section(".stack")))
uint8_t __stack[STACK_SIZE];

__attribute__((section(".vectors")))
voidfunc __vectors[257] = {
  (voidfunc)(&__stack_end),
  __Reset,
  __NMI,
  __HardFault,
  __MemManageFault,
  __BusFault,
  __UsageFault,
  0,
  0,
  0,
  0,
  __SVCall,
  __DebugMonitor,
  0,
  __PendSV,
  __SysTick,
};

void __Reset(){
  uint32_t *src, *dest;
  // Инициализация данных
  src = &__text_end;
  dest = &__data_start;
  while(dest < &__data_end){
    *(dest++) = *(src++);
  }
  // Заполнение bss секции нулями
  dest = &__bss_start;
  while(dest < &__bss_end){
    *(dest++) = 0;
  }
  #if __PLL_CONFIG
  RCC_CR |= _BV(RCC_CR_HSEON);
  while(bit_is_clear(RCC_CR, RCC_CR_HSERDY));
  RCC_CFGR = (0x1 << RCC_CFGR_PLLSRC) | (0x0 << RCC_CFGR_PLLXTPRE) | 
  (0x7 << RCC_CFGR_PLLMUL) | (6 << RCC_CFGR_PPRE1) | 
  (0x3 << RCC_CFGR_ADCPRE);
  RCC_CFGR2 = (0x9 << RCC_CFGR2_PREDIV1) | (0x4 << RCC_CFGR2_PREDIV2) | 
  (0xE << RCC_CFGR2_PLL2MUL) | (0x1 << RCC_CFGR2_PREDIV1SRC);
  RCC_CR |= _BV(RCC_CR_PLL2ON);
  while(bit_is_clear(RCC_CR, RCC_CR_PLL2RDY));
  RCC_CR |= _BV(RCC_CR_PLLON);
  while(bit_is_clear(RCC_CR, RCC_CR_PLLRDY));
  FLASH_ACR |= (2 << FLASH_ACR_LATENCY); // Flash latency = Two wait states, 
  // if 48 MHz < SYSCLK <= 72 MHz
  RCC_CFGR |= (2 << RCC_CFGR_SW); // PLL selected as system clock
  // RCC_CR &= ~(1 << RCC_CR_HSION);
  #endif // __PLL_CONFIG
  #if __RAM_VECT
  VectorTableOffset = (1 << VectorTableOffset_TBLBASE) | 
  (0 << VectorTableOffset_TBLOFF);
  #endif // __RAM_VECT
  main();
  while(true){
    asm("nop"); // main
  }
}

void __Default(){
  while(true){
    asm("nop"); // __Default
  }
}

#pragma weak __NMI = __Default
#pragma weak __HardFault = __Default
#pragma weak __MemManageFault = __Default
#pragma weak __BusFault = __Default
#pragma weak __UsageFault = __Default
#pragma weak __SVCall = __Default
#pragma weak __DebugMonitor = __Default
#pragma weak __PendSV = __Default
#pragma weak __SysTick = __Default

